\version "2.16.0"
%%%
%%% Taken from Nicolas Sceaux
%%%  http://github.com/nsceaux/nenuvar
%%% and adapted to my needs

%% Fixe le colonnage
#(define-markup-command (n-columns layout props lines) (markup-list?)
   #:properties ((column-number 2)
                 (line-width #f)
                 (baseline-skip 0.3)
                 (inter-column-padding 6))
   (let* ((line-width (or line-width (ly:output-def-lookup layout 'line-width)))
          (column-width (/ (- line-width
                              (* (- column-number 1) inter-column-padding))
                           column-number)))
     (let* ((line-stencils
             (space-lines
              baseline-skip
              (map (lambda (line)
                     (interpret-markup
                      layout
                      (cons `((line-width . ,column-width)) props)
                      line))
                   lines)))
            (total-height
             (reduce + 0 (map (lambda (stencil)
                                (interval-length (ly:stencil-extent stencil Y)))
                              line-stencils)))
            (average-height (/ total-height column-number)))
       (let fill-columns ((lines line-stencils)
                          (current-column-index 1)
                          (current-column-height 0)
                          (current-column-lines '())
                          (previous-columns '()))
         (if (null? lines)
             ;; the end result: a single markup with all columns
             (stack-stencil-line
              0
              (reverse! (if current-column-lines
                            (cons (stack-lines
                                   DOWN 0 0
                                   (reverse! current-column-lines))
                                  previous-columns)
                            previous-columns)))
             ;; go on collecting lines into columns
             (let* ((line (car lines))
                    (height (interval-length (ly:stencil-extent line Y))))
               (if (and (> current-column-height 0)
                        (< current-column-index column-number)
                        (>= (+ current-column-height height) average-height))
                   ;; this line ends filling this column
                   ;; => start a new column after it
                   (fill-columns (cdr lines)
                                 (1+ current-column-index)
                                 0
                                 '()
                                 (cons (ly:make-stencil
                                        "" (cons 0 inter-column-padding) '(0 . 0))
                                       (cons (stack-lines
                                              DOWN 0 0
                                              (reverse! (cons line current-column-lines)))
                                             previous-columns)))
                   ;; there is still room in the current column
                   ;; => go on filling it
                   (fill-columns (cdr lines)
                                 current-column-index
                                 (+ current-column-height height)
                                 (cons line current-column-lines)
                                 previous-columns))))))))

%% Mise en forme "Acte"
#(define-markup-command (livretAct layout props text next) (markup? markup?)
   (interpret-markup
    layout props
    (markup #:column (#:fill-line (#:fontsize 4 #:pad-around 3 text)
                      next))))

%% Mise en forme "Scène"
#(define-markup-command (livretScene layout props text next)
     (markup? markup?)
   (stack-lines DOWN 0 0
                (list (ly:make-stencil "" '(0 . 0) '(0 . 1))
                      (interpret-markup
                       layout props
                       (markup #:column (#:fill-line (#:fontsize 2 #:pad-around 2 text)
                                         next))))))

%% Mise en forme "description"
#(define-markup-command (livretDesc layout props text) (markup?)
   (interpret-markup
    layout props
    (markup #:force-line-width-ratio 1/20 #:null
            #:fontsize 0 #:line-width-ratio 9/10 #:pad-around 2 text)))

%% ??
#(define-markup-command (livretDescPage layout props text) (markup?)
   #:properties ((line-width)
                 (gap 12)
                 (word-space 0))
   (interpret-markup
    layout props
    (markup #:hspace gap
            #:override `(line-width . ,(- line-width gap word-space 2))
            #:fontsize 0 #:pad-around 2 text)))

%% ??
#(define-markup-command (livretDescAtt layout props text next)
     (markup? markup?)
   (interpret-markup
    layout props
    (markup #:column
            (#:line (#:force-line-width-ratio 1/20 #:null
                     #:fontsize 1 #:line-width-ratio 9/10 #:pad-around 2 text)
             next))))

%% indication courte à droite italique ([To Albanact.)
#(define-markup-command (livretDidasP layout props text) (markup?)
   (interpret-markup
    layout props
    (markup #:force-line-width-ratio 1/20 #:null
            #:fontsize 0 #:force-line-width-ratio 9/10
            #:fill-line (#:null #:italic text))))

%% indication décalée à droite (par des espaces insécables) italique
%% lorsque plusieurs lignes
#(define-markup-command (livretDidas layout props text) (markup?)
   (interpret-markup
    layout props
    (markup #:fontsize 0 #:italic text)))

#(define-markup-command (livretPers layout props text next) (markup? markup?)
   (interpret-markup
    layout props
    (markup #:column
            (#:fill-line (#:fontsize 1 #:line-width-ratio 7/10
                                     #:pad-around 2 text)
             next))))


%% TRAITEMENT DES VERS
#(define livret-verse-aux
   (let ((gauge-string
          "Qu’en chantant vos feux nous chantions d’autres flâmes ;")
         (gap #f))
     (lambda (layout props verse margin)
       (if (not gap)
           (let ((line-width (chain-assoc-get 'line-width props 0))
                 (gauge (interpret-markup
                         layout props
                         (markup #:fontsize 0 gauge-string))))
             (set! gap (/ (- line-width
                             (interval-length (ly:stencil-extent gauge X)))
                          10.0))))
       (interpret-markup
        layout props
        (markup #:hspace (+ gap margin)
                #:fontsize 0 verse)))))

% vers, indenté (Great Britain's Scepter in great Arthur's hand.)
#(define-markup-command (livretVer layout props args) (markup-list?)
   (livret-verse-aux layout props (make-line-markup args) 4))

% vers, centré
#(define-markup-command (livretVerC layout props args) (markup-list?)
   (livret-verse-aux layout props (make-line-markup args) 2))

% pour aligner réponds à la suite du vers précédent (\invisible"you..."/ ...me)
#(define-markup-command (invisible layout props arg) (markup?)
  (interpret-markup layout props (make-with-color-markup white arg)))

%% rôle suivi du vers (\italic "Conon. " Then this is the deciding day, to fix)
#(define-markup-command (livretPersVerse layout props args) (markup-list?)
   (livret-verse-aux layout props (make-line-markup args) 0))


%%Mise en forme chant
#(define livret-lyric-aux
   (let ((gauge-string
          "The juice that makes the Britons bold.")
         (gap #f))
     (lambda (layout props lyric margin)
       (if (not gap)
           (let ((line-width (chain-assoc-get 'line-width props 0))
                 (gauge (interpret-markup
                         layout props
                         (markup #:fontsize 0 gauge-string))))
             (set! gap (/ (- line-width
                             (interval-length (ly:stencil-extent gauge X)))
                          3.5))))
       (interpret-markup
        layout props
        (markup #:hspace (+ gap margin)
                #:fontsize 0 lyric)))))

%% rôle suivi de lyric (\italic "Chor.  " We have sacrificed.)
#(define-markup-command (livretPersLyric layout props args) (markup-list?)
   (livret-lyric-aux layout props (make-line-markup args) -3))

% ligne de chant indenté (Woden, first to thee,)
#(define-markup-command (livretLyr layout props args) (markup-list?)
   (livret-lyric-aux layout props (make-line-markup args) 0))
